class WhatifyResponse(object):
    def __init__(self, data: dict = None, headers: dict = None, status_code: int = None):
        self.headers = headers or {}
        self._data = data or {}
        self.status_code = status_code

    def __getitem__(self, key):
        return self._data.get(key, None)

    def __repr__(self):
        return str(self._data)

    def __contains__(self, item):
        return item in self._data

    def get(self, key, default=None):
        return self._data.get(key, default)

    def to_dict(self):
        return self._data

    @property
    def request_id(self):
        return self.headers.get("X-Request-ID", None)



"""
Wisdom is the user interaction with Whatify wisdom, it contains

"""
# Prediction is the way to productize the Ensemble created in the previous steps. Once an Ensemble is created,
# users can upload additional Datasources that may be used for predictions.
#
# ‘Prediction’ API includes querying of predictions (Get, List and Delete) and creating a Prediction to get predictions
# on existing Ensembles and uploaded Datasources.
import os

import s3fs
from typing import Dict
import toolkit_w
from toolkit_w.internal.api_requestor import APIRequestor
from toolkit_w.internal.errors import APIError
from toolkit_w.internal.whatify_response import WhatifyResponse
from toolkit_w.resources.api_resource import APIResource


class UserManagement(APIResource):
    _CLASS_PREFIX = ''

    @classmethod
    def list(cls, url: str='users/list', search_term: str = None, page: int = None, page_size: int = None, sort: Dict = None,
             filter_: Dict = None, api_key: str = None) -> WhatifyResponse:
        """
        List the existing Tasks - supports filtering, sorting and pagination.

        Args:
            search_term (Optional[str]): Return only records that contain the `search_term` in any field.
            page (Optional[int]): For pagination, which page to return.
            page_size (Optional[int]): For pagination, how many records will appear in a single page.
            sort (Optional[Dict[str, Union[str, int]]]): Dictionary of rules  to sort the results by.
            filter_ (Optional[Dict[str, Union[str, int]]]): Dictionary of rules to filter the results by.
            api_key (Optional[str]): Explicit `api_key`, not required, if `fireflyai.authenticate()` was run prior.
            :param url: the API url of users\accounts

        Returns:
            WhatifyResponse: users are represented as nested dictionaries under `hits`.
        """
        return cls._list(url=url, search_term=search_term, page=page, page_size=page_size, sort=sort, filter_=filter_, api_key=api_key)

    @classmethod
    def login(cls, email: str, password: str) -> WhatifyResponse:
        """
        Authenticates user and stores temporary token in `toolkit_w.token`.

        Other modules automatically detect if a token exists and use it, unless a user specifically provides a token
        for a specific request.
        The token is valid for a 24-hour period, after which this method needs to be called again in order to generate
        a new token.

        Args:
            email (str): email.
            password (str): Password.

        Returns:
            WhatifyResponse: Empty WhatifyResponse if successful, raises FireflyError otherwise.
        """
        url = 'login'

        requestor = APIRequestor()
        response = requestor.post(url, body={'username': email, 'password': password, 'tnc': None}, api_key="")
        toolkit_w.token = response['token']
        print(' '.join(['user mail:', str(email), '- Login successful']))

        return WhatifyResponse(status_code=response.status_code, headers=response.headers)

    @classmethod
    def impersonate(cls, user_id: str = None, email: str = None, admin_token: str = None) -> WhatifyResponse:
        """
       impersonate user and stores temporary token in `toolkit_w.token`

       Args:
           user_id (str): user ID.
           email (str): User email.
           admin_token (str): Admin user token.

       Returns:
           WhatifyResponse: Empty WhatifyResponse if successful, raises WhatifyError otherwise.
       """
        if email:
            user_id = None
            raise Exception('TODO: need tom implement get user ID from mail')

        url = ''.join(['users/login_as/', str(user_id)])
        requester = APIRequestor()
        response = requester.post(url, api_key=admin_token)
        toolkit_w.token = response['result']
        print(' '.join(['user ID:', str(user_id), '- Login successful']))
        return WhatifyResponse(status_code=response.status_code, headers=response.headers)

    @classmethod
    def get_user_list(cls, username: str, admin_token: str = None) -> WhatifyResponse:
        ret_val = {}
        resp = cls.list(filter_={'username': [username]}, api_key=admin_token)
        if resp and 'result' in resp and len(resp['result']) == 1:
            ret_val = resp['result'][0]
        return ret_val

    @classmethod
    def get_account_list(cls, name: str, admin_token: str = None) -> WhatifyResponse:
        ret_val = {}
        resp = cls.list(url='accounts', filter_={'name': [name]}, api_key=admin_token)
        if resp and 'result' in resp and len(resp['result']) == 1:
            ret_val = resp['result'][0]
        return ret_val

    # @classmethod
    # def test_API(cls, var_str: str, user_token: str = None) -> WhatifyResponse:
    #     print('aa' + str(var_str))
    #     # cls._list(search_term='auto_qa_admin')
    #     # url = ''.join(['users/login_as/', str(6206)])
    #     # url = ''.join(['accounts'])
    #     url = 'reports/predictions/25568/insights/summary'
    #     requester = APIRequestor()
    #     try:
    #         response = requester.get(url, api_key=user_token)
    #     except APIError as ex:
    #         print((str(ex)))
    #
    #     response = requester.post(url, api_key=user_token)
    #     print('sss')
    #     print('sss')
    #     print('sss')
    #     print('sss')




import logging
from logging import NullHandler
# Set default logging handler to avoid "No handler found" warnings.
from toolkit_w.internal import vars_global
from toolkit_w.resources import *
from toolkit_w.whatify_toolkit import MyWhatify

logger = logging.getLogger(__name__)
token = None
user_token = None
api_base = 'https://api.firefly.ai'





"""
Wisdom is the user interaction with Whatify wisdom, it contains

"""
# Prediction is the way to productize the Ensemble created in the previous steps. Once an Ensemble is created,
# users can upload additional Datasources that may be used for predictions.
#
# ‘Prediction’ API includes querying of predictions (Get, List and Delete) and creating a Prediction to get predictions
# on existing Ensembles and uploaded Datasources.
import os
from typing import Dict, List

from toolkit_w.internal import utils
from toolkit_w.internal.api_requestor import APIRequestor
from toolkit_w.internal.whatify_response import WhatifyResponse
from toolkit_w.resources.api_resource import APIResource


class Wisdom(APIResource):
    _CLASS_PREFIX = 'foresights'

    @classmethod
    def get_wisdom_list_with_demo(cls, api_key: str = None) -> WhatifyResponse:
        """
        Gets the wisdoms list for the user with the demo wisdoms.

        Args:
            api_key (Optional[str]): Explicit `api_key`, not required, if `Whatify.login()` was run prior.

        Returns:
            WhatifyResponse: Contains mapping of wisdoms for the user.
        """
        requestor = APIRequestor()
        url = '{prefix}/with_demo'.format(prefix=cls._CLASS_PREFIX)
        response = requestor.get(url, api_key=api_key)['hits']
        return response

    @classmethod
    def get_wisdom(cls, id: int, api_key: str = None) -> WhatifyResponse:
        """
        Gets the wisdom data for the user by its ID

        Args:
            id (int): wisdom (foresight) ID.
            api_key (Optional[str]): Explicit `api_key`, not required, if `Whatify.login()` was run prior.

        Returns:
            WhatifyResponse: Contains wisdom data by ID for the user.
        """
        requestor = APIRequestor()
        url = '{prefix}/{id}'.format(prefix=cls._CLASS_PREFIX, id=id)
        response = requestor.get(url, api_key=api_key)
        return response

    @classmethod
    def delete(cls, id: int, api_key: str = None) -> WhatifyResponse:
        """
        Deletes a specific Wisdom.

        Args:
            id (int): Wisdom ID.
            api_key (Optional[str]): Explicit `api_key`, not required, if `Whatify.login()` was run prior.

        Returns:
            WhatifyResponse: "true" if deleted successfuly, raises WhtifyClientError otherwise.
        """
        return cls._delete(id, api_key)


